#!/usr/bin/env python

import sys
import re
import math

def usage():
    print """python script.py L1_FREQ L2_FREQ ALIGN_FREQ
The scripts reads frequency lists for L1 and L2 languages
and frequency list of co-occurrences of items from L1 and L2
and computes bilingual dictionary which is written to standard output."""

def parse_freq(filename):
    "read frequency list and save into Python dictionary"
    d = {}
    for line in open(filename):
        try:
            l = line.decode('utf-8')
            f, w = l.split()
            if int(f) > 0:
                # TODO
                # use more strict filtering?
                d[w] = int(f)
        except:
            continue
    return d

def get_score(l1f, l2f, l12f):
    # here logdice is used, other co-occurrence statistics might be used
    # TODO
    if (l1f + l2f) == 0:
        return - float('inf')
    return math.log(float(l12f) * 2 / (l1f + l2f), 2)

def compute_dict(l1, l2, aligned):
    l1fd = parse_freq(l1)
    l2fd = parse_freq(l2)
    rexp = re.compile('\s*([0-9]+)\s*(.*)$')
    result = {}
    for line in open(aligned):
        l = line.decode('utf-8').strip()
        m = re.match(rexp, l)
        # read freqs from uniq -c output
        if m:
            l12f = int(m.group(1))
            l1w, l2w = m.group(2).split('\t')
            result.setdefault(l1w, [])
            l1f = l1fd.get(l1w, 0)
            l2f = l2fd.get(l2w, 0)
            if not l12f:
                continue
            if l1f and l2f:
                score = get_score(l1f, l2f, l12f)
                result[l1w].append((l2w, l1f, l2f, l12f, score))
    for k in result:
        # sort by score (logdice)
        # TODO
        translations = sorted(result[k], key=lambda x: x[4], reverse=True)
        # take only top X, or use a heuristics
        # TODO
        for i in range(min(2, len(translations))):
            print '%s\t%s\t%.5f' % (k.encode('utf-8'), translations[i][0].encode('utf-8'), translations[i][4])

if __name__ == '__main__':
    if len(sys.argv) != 4:
        usage()
        exit(1)
    compute_dict(sys.argv[1], sys.argv[2], sys.argv[3])
